﻿using SharpDX;
using System;
using System.Collections.Generic;
using System.Linq;
using VL.Core;
using VL.Lib.Collections;


namespace VL.Dijkstra
{
    [Type]
    public class Graph
    {

        [Node]
        public Spread<Vertex> Vertices { get; private set; }

        [Node]
        public Graph(IEnumerable<Vector2> positions, IEnumerable<Vector2> connections)
        {
            Vertices = null;
            Update(positions, connections);
        }

        [Node]
        public IEnumerable<Vertex> Update(IEnumerable<Vector2> vertexPositions, IEnumerable<Vector2> connections, IEnumerable<int> blocked = null)
        {
            if (blocked == null) blocked = Enumerable.Empty<int>();

            var positions = vertexPositions.ToList();
            var newVertices = new Vertex[positions.Count];

            var counter = 0;
            foreach (var pos in positions)
            {
                newVertices[counter] = new Vertex(counter, pos);
                counter++;
            }

            foreach (var connection in connections.ToList())
            {
                var start = newVertices[(int)connection.X];
                var end = newVertices[(int)connection.Y];
                
                var dist = Vector2.Distance(start.Position, end.Position);

                if (!blocked.Contains(start.Index) && !blocked.Contains(end.Index))
                    start.AddNeighbour(end, dist);
            }

            Vertices = Spread.Create(newVertices);
            return Vertices;
        }

        [Node]
        public override string ToString()
        {
            return "Vertices: " + Vertices.Count + "\nEdges: " + Vertices.Select(v => v.Neighbours.Count).Sum();
        }

    }

}
